#### PPunCC model v5.7 implemented as R functions
##Copyright (C) 2013  Michael Renton

##    This program is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.

##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.

##    You should have received a copy of the GNU General Public License
##    along with this program.  If not, see <http://www.gnu.org/licenses/>.

##	For further information on this model, please contact Michael Renton
##	The University of Western Australia, M090, Stirling Highway, WA, Australia
##	michael.renton@uwa.edu.au

##	This version of PPunCC should be cited by reference to this paper in Env Con
##	and the two earlier papers in Global Change Biol  and  Ecol Mod  which provide full
##	background details

######################################################################################
######################################################################################
######################################################################################

#### the model can be run for the default set of parameter values simply by running this script
#### of if the last line is commented out, then this script provides a function that can be called to conduct analyses
#### the main function is 'dosim' and the required argument is a set of parameter values (see below)
#### two other required functions are also defined first, as per commnents below

#### to start with, the user may be most interested in the lines at the bottom 
#### these modify parameters, select landscapes and execute the simulation 

######################################################################################
######################################################################################
######################################################################################
## this function precalculates values to improve efficiency of dispersal simulation

getdistproblist <- function(param,ifFat){

distlist <- 0
xc <- 0
yc <- 0
prob <- 0
for (dx in seq(-0.495,0.495,by=0.01)){
	for (dy in seq(-0.45,0.45,by=0.1)){
		pt <- c(xc+dx,yc+dy)
		dist <- sqrt(sum(pt^2))
		if (ifFat)	den <- dweibull(dist,shape=0.5,scale=param)/(2*pi*dist^2)
		else den <- dexp(dist,rate=param)/(2*pi*dist^2)
		prob <- prob + den
	}
}
problist <- prob
total <- prob
nsq <- 25
for (xc in 0:nsq ){
for (yc in 1:nsq ){
	#print(paste(xc,yc))
	celldist <- sqrt(xc^2+yc^2)
	if (celldist %in% distlist){
		prob <- problist[which(distlist==celldist)[1]]
	} else {
		prob <- 0
		for (dx in seq(-0.495,0.495,by=0.01)){
			for (dy in seq(-0.45,0.45,by=0.1)){
				pt <- c(xc+dx,yc+dy)
				dist <- sqrt(sum(pt^2))
				if (ifFat)	den <- dweibull(dist,shape=0.5,scale=param)/(2*pi*dist^2)
				else den <- dexp(dist,rate=param)/(2*pi*dist^2)
				prob <- prob + den
			}
		}
		distlist <- c(distlist,celldist)
		problist <- c(problist,prob)
	}
	total <- total + 4 * prob
	}
}
problist <- problist / total
problist <- problist[distlist <=nsq]
distlist <- distlist[distlist <=nsq]
plot(distlist,problist)
lines(distlist,problist)
cbind(distlist,problist)
}

######################################################################################
## this function sets the parameter values and returns an objext to be passed to the main simulation function
## values in this object can be changed in loops to facilitate automation of more extensive analyses

initialiseparamvallist <- function(){
	paramvallist <- list(
			landscapefile=NULL,  ## if this remains as NULL, then a random landscape is created 
			fragmentation=0.8,  ## fragmentation IF a random landscape is created, pf in paper
			climatechangerate=-1 ,  ## rcc in paper
			envtolerance=4 ,   ## ctol in paper
			yearofmaturity=4 ,   ## Ym in paper
			deathage=100 ,   ## not used in paper, by setting big ensures has no effect since stochatsic mortality will overide  
			optimalexpectedrecruitmentperplant=100 , ## F in paper
			D=100 , ## D in paper - note this is per ha, and gets converted to per cell later
			chsurv=0.9 , ## 1-pm in paper
			Tpc=10.25*log(100)/log(100*100*100) ,   ## this is T in paper
			ifFat=FALSE , 
			showplots =TRUE
	)
}



######################################################################################
## this function conducts one full simulation run
## it takes as argument a list of parameter values, as generated by the previous function
## and simply returns TRUE/FALSE indicating whether the simulated species persisted or not...
## ...under the simulated conditions, in the simulated landscape 

dosim <- function(paramvallist ){

### initialise these passed parameters

landscapefile <- paramvallist$landscapefile
fragmentation <- paramvallist$fragmentation
climatechangerate <- paramvallist$climatechangerate 
envtolerance <- paramvallist$envtolerance 
yearofmaturity <- paramvallist$yearofmaturity 
deathage <- paramvallist$deathage 
optimalexpectedrecruitmentperplant <- paramvallist$optimalexpectedrecruitmentperplant
D <- paramvallist$D
chsurv <- paramvallist$chsurv 
Tpc <- paramvallist$Tpc 
ifFat <- paramvallist$ifFat
showplots <- paramvallist$showplots
if (is.null(paramvallist$vulpt)) paramvallist$vulpt <- 1
vulpt <- paramvallist$vulpt
if (is.null(paramvallist$climvar)) paramvallist$climvar <- 0
climvar <- paramvallist$climvar
if (is.null(paramvallist$climstepsize)) paramvallist$climstepsize <- 1
climstepsize <- paramvallist$climstepsize 
if (is.null(paramvallist$probextreme)) paramvallist$probextreme <- 0
probextreme <- paramvallist$probextreme
if (is.null(paramvallist$typeextreme)) paramvallist$typeextreme <- 1
typeextreme <- paramvallist$typeextreme

#### world parameters

gradlength <- 100
gradwidth <- 20
gradmin <- 500
gradmax <- 800
maxt <- 400
climatechangestart <- 30
climatechangefinish <- 30 + 270 / -climatechangerate 
link <- FALSE
maxage <- max(yearofmaturity+1,2)
dieatmax <- FALSE

### species parameters
optenv <- 510
optDen <- round(D*100)
FD <- optimalexpectedrecruitmentperplant*D
distT <- Tpc * log(FD) / log(100)         #### this distT is Tn in the paper - no longer used in this model
 
## dispersal is modelled with an exponetial distribtuion or a weibull distribution (ifFat)
k <- log(100)/Tpc
lambda <- Tpc/(log(100))^2
param <- k
if (ifFat) param <- lambda
print(param)
distproblist <- NULL
distproblistfname <- paste("distproblist",param,ifFat,".dplst")
if (distproblistfname %in% list.files()){
	load(distproblistfname)
} else {
	print("calculating distproblist")
	distproblist <- getdistproblist(param,ifFat)
	save(distproblist ,file=distproblistfname )
}
maxdistinlist <- max(distproblist[,1])

#### set up world
par(mfrow=c(3,1))
celllength <- 1
ncellslong <- gradlength/celllength 
ncellswide <- gradwidth /celllength 
ncells <- ncellswide * ncellslong
env <- gradmax - c(1:ncellslong )/ncellslong *(gradmax-gradmin)
envarray <- t(array(rep(env,ncellswide ),dim = c(ncellslong ,ncellswide)))
if (showplots ) {image(x=1:ncellslong , y=1:ncellswide , z = t(envarray),xlab = "",ylab="")}
number <- array(rep(0,ncellswide * ncellslong*maxage),dim=c(ncellswide , ncellslong,maxage))
#babies <- array(rep(0,ncellswide * ncellslong),dim=c(ncellswide , ncellslong))
fragmentarray <- 1
if (is.null(landscapefile)){
	nfrags <- round(fragmentation*ncells)
	fragmentvec <- sample(ncells,nfrags )
	fragmentvec2 <- rep(0, ncells)
	fragmentvec2[fragmentvec] <- 1
	fragmentarray <- array(fragmentvec2,dim = c(ncellslong,ncellswide  ))
} else {
	load(landscapefile)
	fragmentarray[fragmentarray<1] <- 0
}
fragmentarray <- t(fragmentarray)
### note that 1 in fragmentarray is unsuitable
if (link ) fragmentarray[,round(ncellswide /2)] <- 0
if (showplots ) {image(x=1:ncellslong , y=1:ncellswide , z = t(fragmentarray) ,xlab = "",ylab="")}


### work out env suitability for initialising population
kkk <- 1/(envtolerance)^2
envsuitability <- 1/(1+kkk*(envarray-optenv)^2)  ##dnorm(envarray ,optenv,envtolerance)
image(x=1:ncellslong , y=1:ncellswide , z = t(envsuitability) ,xlab = "",ylab="")
seednums <- round(optDen/maxage)
number[,,] <- seednums
pseed <- envsuitability-0.3
pseed[pseed<0] <- 0
number[,,] <- floor(number[,,] * rep(pseed,maxage) *0.5)

number[fragmentarray==1 ]<-0
if (showplots ) {image(z = t(number[,,2]),xlab = "",ylab="")}
if (showplots ) {plot(c(1,2),c(2,3))}

print("starting new simulation.......")
print(c('VALS',landscapefile ,optimalexpectedrecruitmentperplant ,D,yearofmaturity ,deathage , chsurv , Tpc))

### run simulation
t <- -20
sumvec <- NULL
allsum <- c(0)
#print("start")
#print(number)
endzone <- 0
climrec <- NULL
while (t <= maxt & sum(number)>optDen & !(t>100 & sum(number)<optDen) & endzone < 11){
	t <- t+1
	#print(t)
	if (t>climatechangestart & t<climatechangefinish) {
		if (t%%climstepsize==0) {
			envarray <- envarray + climatechangerate * climstepsize
			#print (paste("climate changing",mean(envarray)))
		}
	}
	baseenvarray <- envarray    ## make a copy so climate can vary
	envarray <- envarray + rnorm(1,0,climvar)
	climrec <- c(climrec,mean(envarray ))
	envsuitability <- 1/(1+kkk*(envarray-optenv)^2)
	#if (t==0) {number[number<seednum] <- 0}
	#print(paste("time:",t))
	#print(number)
		### because germination and establishemnt are inlcuded in parameters
		### all year 1 plants are assumed to germinate and establish by this point
		thisnumbersummed <- rowSums(number,dims=2)
		if (showplots ) {image(x=1:ncellslong , y=1:ncellswide , 
				z = t(1-fragmentarray) * optDen + t(thisnumbersummed)  ,
				zlim = c(0,2*optDen),xlab=paste("time:",t),ylab="")}
		#readline()# 
		### and now seed set and dispersal
		matureplants <- rowSums(number[,,(yearofmaturity:maxage)],dims=2)
		#image(x=1:ncellslong , y=1:ncellswide , z = t(matureplants )  ,zlim = c(0,2*optDen),xlab=paste("time:",t),ylab="")
		optimalexpectedrecruitmentperplantX <- optimalexpectedrecruitmentperplant 
		if (vulpt==3) optimalexpectedrecruitmentperplantX <- optimalexpectedrecruitmentperplant * envsuitability 
		if (vulpt==4) optimalexpectedrecruitmentperplantX <- optimalexpectedrecruitmentperplant * envsuitability^(0.5) 
		expectedbabies <- optimalexpectedrecruitmentperplantX*matureplants
		if (probextreme>runif(1) & typeextreme==3) expectedbabies <- expectedbabies * 0.6
		if (probextreme>runif(1) & typeextreme==4) expectedbabies <- expectedbabies * 0.2
		if (probextreme>runif(1) & typeextreme==8) expectedbabies <- expectedbabies * 0.2
		if (probextreme>runif(1) & typeextreme==5) expectedbabies <- expectedbabies * 0.01
		thisbabiesmade <- rpois(ncells ,expectedbabies )
		thisbabiesmade[is.nan(thisbabiesmade )] <- 0
		thisbabiesmade <- array(thisbabiesmade , dim=c(ncellswide , ncellslong))
		thisbabiesarrive <- array(rep(0,ncellswide * ncellslong),dim=c(ncellswide,ncellslong))
		suitablecells <- which(fragmentarray==0 & envsuitability > 0.01,arr.ind = TRUE)
		babycells <- which(thisbabiesmade>0,arr.ind = TRUE)
		nbc <- length(babycells[,1]); ndc <- length(suitablecells[,1])
		if (nbc*ndc > 0) {
		for (destcellind in 1:ndc){
			for (babcellind in 1:nbc){
				thisdist <- sqrt(sum((suitablecells[destcellind,] - babycells[babcellind,])^2))
				narrivals <- 0
				if (thisdist <= maxdistinlist ){
					ind <- which(distproblist[,1]==thisdist)
					#if (length(ind)==0) print(thisdist) #else print("ok")
					probarr <- distproblist[,2][ind]
					narrivals <- rbinom(1,thisbabiesmade[babycells[babcellind,1],babycells[babcellind,2]],probarr )
				#print("indices")
				#print(suitablecells[destcellind,])
				#print(babycells[babcellind,])
				#print(paste(probarr , thisbabiesmade[babycells[babcellind,]], narrivals ))
					if (is.na(narrivals)) print(probarr )
				}
				thisbabiesarrive[suitablecells[destcellind,1],suitablecells[destcellind,2]] <- thisbabiesarrive[suitablecells[destcellind,1],suitablecells[destcellind,2]] + narrivals
			}
		}
		}
		### now death due to random chance or age (or extreme event)
		if (vulpt==2) {chsurvX <- chsurv*envsuitability^(0.25)} else {chsurvX <- chsurv}
		if (probextreme>runif(1) & typeextreme==1) chsurvX <- chsurvX * 0.6
		if (probextreme>runif(1) & typeextreme==8) chsurvX <- chsurvX * 0.6
		if (probextreme>runif(1) & typeextreme==2) chsurvX <- chsurvX * 0.2
		number[,,] <- array(rbinom(length(number[,,]),number[,,],chsurvX),dim=dim(number[,,]))
		if (deathage<maxage) {number[,,deathage:maxage] <- 0}
	#print("after death")
	#print(thisbabiesarrive )
		### now work out germination and establishment for next year
		envsuitability <- 1/(1+kkk*(envarray-optenv)^2)
		#image(x=1:ncellslong , y=1:ncellswide , z = t(envsuitability) ,xlab = "",ylab="")
		pestab <- 1
		if (vulpt==1) {pestab <- envsuitability }
		if (vulpt==4) {pestab <- envsuitability^(0.5) }
		if (probextreme>runif(1) & typeextreme==6) pestab <- pestab * 0.6
		if (probextreme>runif(1) & typeextreme==7) pestab <- pestab * 0.2
		if (probextreme>runif(1) & typeextreme==8) pestab <- pestab * 0.2
		thisbabiesestablish <- rbinom(ncells, thisbabiesarrive , pestab ) ## babies establish according to environemntal suitability (or not)
		thisbabiesestablish [is.nan(thisbabiesestablish )] <- 0
		thisbabiesestablish <- array(thisbabiesestablish , dim=c(ncellswide , ncellslong))
		#image(x=1:ncellslong , y=1:ncellswide , z = t(thisbabiesestablish ) ,xlab = "",ylab="")
		oldest <- number[,,maxage]
		number[,,2:maxage] <- number[,,1:(maxage-1)] ## age all plants one year
		number[,,1] <- 0
		if (!dieatmax) number[,,maxage] <- number[,,maxage] + oldest 
		thisnumbersummed <- rowSums(number[,,],dims=2)
		freespace <- optDen- thisnumbersummed ### constrain grmination/establishment by capacity
		thisbabiesestablish [thisbabiesestablish >freespace] <- freespace[thisbabiesestablish >freespace]
		thisbabiesestablish [fragmentarray==1 | thisbabiesestablish < 0] <- 0
		#image(x=1:ncellslong , y=1:ncellswide , z = t(thisbabiesestablish )  ,xlab=paste("time:",t),ylab="")
		number[,,1] <- thisbabiesestablish 
	#print("after estab")
	#print(number)
	#print(thisbabiesestablish )
		#if (any(is.nan(number[,,sp,]))){print(paste("nan",length(number[,,sp,]),number[,,sp,],chsurv[sp]))}
	#}
	numberacrossgrad <- apply(number,2,sum)
	endzone <- sum(numberacrossgrad[1:20])
	if (showplots ) {plot(log(numberacrossgrad+1),ylim=c(0,log(3*optDen[1]*ncellswide*(1-fragmentation))),t='l',xlab="")}
	if (t>0) {allsum <- c(allsum,sum(number))}
	if (showplots ) {plot(allsum ,xlab='time',xlim=c(0,maxt ))}
	#for (age in 1:maxage){print(paste(sum(number[,,age]),"total of age",age))}
	#print("end of step")
	#print(number)
	#readline()
	envarray <- baseenvarray    ## set it back to variance is not cumulative
}
persist <- FALSE
if (t>=maxt | endzone>10){persist <- TRUE}
print(c("persist:",persist,"endzone",endzone,"sum(number)",sum(number),"optDen",optDen))
#print("end")
#print(number)
plot(climrec)
persist 
}

#############################################################################################
#############################################################################################
#############################################################################################
## set up and execute the run



### set up the possible functional types and landscapes
landscapelist <- c("newlandscape80r1.5","landscape80","newlandscape95r1.5","landscape95") 
lifecycles <- data.frame(Ym=c(1,4,10),Pm=c(1,0.1,0.04),F=c(100,1000,10000),TTT=c(1.26,1.4,2.26))

### choose which ones to use
lsci <- 2
lc <- 2

### set up a default list of parameter values
paramvallist <- initialiseparamvallist() 

### modify according to choices above
paramvallist$landscapefile<-landscapelist[lsci]
print(paramvallist$landscapefile)
paramvallist$optimalexpectedrecruitmentperplant <- lifecycles$F[lc]
paramvallist$chsurv <- 1-lifecycles$Pm[lc]
paramvallist$yearofmaturity <- lifecycles$Ym[lc]
paramvallist$Tpc <- lifecycles$TTT[lc]

######## climate parameter values can be changed here 
#paramvallist$climstepsize <- 10
#paramvallist$climvar <- 10
#paramvallist$probextreme<- 0.5
#paramvallist$typeextreme<- 8

## do the simulation
ifpersist <- dosim (paramvallist)
print(paste('Persist:',ifpersist))



